<?php

namespace DataBase;

/**
 * @internal
 */
final class IndexCollection implements IndexCollectionInterface
{
    /**
     * @var IndexInterface[]
     */
    private $indexes;

    /**
     * IndexCollection constructor.
     *
     * @param IndexInterface[] $indexes
     */
    public function __construct(IndexInterface ...$indexes)
    {
        $this->indexes = $indexes;
    }

    /**
     * {@inheritdoc}
     */
    public function getUnique()
    {
        return $this->getFiltered(function (IndexInterface $index) {
            return $index->isUnique();
        });
    }

    /**
     * @param callable $filter
     *
     * {@inheritdoc}
     */
    public function getFiltered(callable $filter)
    {
        return new self(...\array_filter($this->asArray(), $filter));
    }

    /**
     * @return IndexInterface[]
     */
    public function asArray()
    {
        return $this->indexes;
    }

    /**
     * @inheritdoc
     * @return \ArrayIterator|IndexInterface[]
     */
    public function getIterator()
    {
        return new \ArrayIterator($this->asArray());
    }
}