<?php

namespace DataBase;

/**
 * @internal
 */
final class MySqlIndex implements IndexInterface
{
    /**
     * @var array[]
     */
    private $columnsInIndex;

    /**
     * Index constructor.
     *
     * @param array[] $columnsInIndex
     */
    public function __construct(array $columnsInIndex)
    {
        if (count($columnsInIndex) === 0) {
            throw new \InvalidArgumentException();
        }

        $this->columnsInIndex = $columnsInIndex;
    }

    public function hasColumn($columnName)
    {
        return array_reduce(
            $this->columnsInIndex,
            function ($hasColumn, array $columnInIndex) use ($columnName) {
                return $hasColumn || $columnInIndex['column_name'] === $columnName;
            },
            false
        );
    }

    public function isUnique()
    {
        return $this->columnsInIndex[0]['non_unique'] == 0;
    }

    /**
     * {@inheritdoc}
     */
    public function matchesData(array $data)
    {
        foreach ($this->columnsInIndex as $columnInIndex) {
            $columnName = $columnInIndex['column_name'];
            if (!array_key_exists($columnName, $data) || $data[$columnName] === null) {
                return false;
            }
        }

        return true;
    }

    /**
     * @param array $data
     *
     * @return array
     */
    public function getMatchingData(array $data)
    {
        if (!$this->matchesData($data)) {
            throw new \BadMethodCallException();
        }

        return array_filter(
            $data,
            function ($columnName) {
                return $this->hasColumn($columnName);
            },
            ARRAY_FILTER_USE_KEY
        );
    }
}