<?php

namespace DataBase;

/**
 * @internal
 */
final class MysqlIndexCollection implements IndexCollectionInterface
{
    use IndexCollectionDecoratorTrait;

    /**
     * @var array[]
     */
    private $rows;

    /**
     * @var IndexCollection|null
     */
    private $collection;

    /**
     * MysqlIndexCollection constructor.
     *
     * @param $columnsInIndexes
     */
    public function __construct(array $columnsInIndexes)
    {
        $this->rows = $columnsInIndexes;
    }

    protected function getCollection()
    {
        if ($this->collection === null) {
            $this->loadCollection();
        }

        return $this->collection;
    }

    /**
     *
     */
    private function loadCollection()
    {
        $rows = $this->getNormalisedColumnsInIndex();

        $groupedColumns = \array_reduce(
            $rows,
            function (array $indexes, array $row) {
                $keyName = $row['key_name'];
                $indexes += [$keyName => []];
                $indexes[$keyName][] = $row;

                return $indexes;
            },
            []
        );

        $indexes = \array_values(
            \array_map(
                function (array $columnsInIndex) {
                    return new MySqlIndex($columnsInIndex);
                },
                $groupedColumns
            )
        );

        $this->collection = new IndexCollection(...$indexes);
    }

    /**
     * @return array
     */
    private function getNormalisedColumnsInIndex()
    {
        $rows = \array_map(
            function (array $columnInIndexes) {
                return \array_change_key_case($columnInIndexes, CASE_LOWER);
            },
            $this->rows
        );

        return $rows;
    }
}